// festival2phone.c  version 1.2
// 
// simple program to convert words into their phonemes
// the program uses festival to do the hard work
// to compile execute "gcc festival2phone.c -o festival2phone"
// Festival should be configured to use the required language
// (this is normally done in file /usr/share/festival/voices.scm
// in the variable defvar default-voice-priority-list)
// This program is under GNU license
// Developed by ubanov for VoxForge (2008)

#include <stdio.h>
#include <string.h>
#include <ctype.h>

// if you want to construct a filter (where you give a list of words and
// program call festival to give all the pronemes)
#define FILTER 0

// print only the phonemes of the words (can be used to obtain the full
// list of the phonemes with their probability)
#define ONLYPRINTPHONEMES 1

// program to execute festival (doesn't work with festival_client, then only
// can have the value festival (or may be the full path to festival)
#define FESTIVAL "festival"



int QuitSpecialChars(char *word)
{
  char *especial=",.;:?!¿¡";
  char *punt;

  if(word==NULL)
    return -1;
  punt=word;
  while(*word) {
    if(strchr(especial,*word)!=NULL)
      *word=' ';
    else
      *word=tolower(*word);
    word++;
  }
  while(*punt==' ')
    memmove(punt,punt+1,strlen(punt));
  if(strchr(punt,' '))
    *strchr(punt,' ')=0;
  return 0;
}


int Festival2Lex(char *word,FILE *sal)
{
  char buf[1024],buf2[1024],*punt;
  FILE *fich;

  if(sal==NULL)
    sal=stdout;
  sprintf(buf,"echo \"(lex.lookup \\\"%s\\\")\" | %s -i --pipe",word,FESTIVAL);
  if((fich=popen(buf,"r"))==NULL)   /* execute festival with a pipe */
    return -1;

  buf2[0]=0;	/* init buf2 to empty */

  fgets(buf,sizeof(buf),fich);    /* get first line */
  while(!feof(fich)) {		/* while there are more lines... */
    if(strchr(buf,'>') && buf2[0]==0)	/* store first line containing > char */
      strcpy(buf2,strchr(buf,'>')+1);
    if(buf2[0] && !strchr(buf,'>'))	/* if is a long word, its divided in */
      strcat(buf2,buf);		/* several lines, append all the lines */
    fgets(buf,sizeof(buf),fich);	/* get next line */
  }
  pclose(fich);			/* close the program and the pipe */
  if(buf2[0]==0)	/* if there is no line with '>', exit */
    return -2;
  while(strchr(buf2,'\n')) /* if it's multiline change returns by spaces */
    *strchr(buf2,'\n')=' ';
  if(!strchr(buf2,'('))		/* search first ( */
    return -3;
  punt=strchr(buf2,'(')+1;
  if(!strchr(punt,'('))		/* search second ( */
    return -3;
  punt=strchr(punt,'(')+1; /* now punt points to ((p r o) 0) ((b a1 n) 1)) */
  while(*punt!=')') {
    punt+=2;
    while(*punt!=')') {		/* printf silabe */
      fprintf(sal,"%c",*punt);
      if(ONLYPRINTPHONEMES && *punt==' ')
        fprintf(sal,"\n");
      punt++;
    }
    if(ONLYPRINTPHONEMES)	/* printf end of the silabe */
      fprintf(sal,"\n");
    else
      fprintf(sal," ");
    if(strchr(punt+1,')'))	/* search the end of the silabe */
      punt=strchr(punt+1,')')+1;
    while(*punt==' ')		/* skip spaces, until next valid character */
      punt++;
  }

  return 0;
}



int dofilter(void)
{
  char buf[1024],word[256];
  int nerr;
  
  fgets(buf,sizeof(buf),stdin);
  while(!feof(stdin)) {
    sscanf(buf,"%s",word);
    QuitSpecialChars(word);
    if(!ONLYPRINTPHONEMES)
      printf("%s\t[%s]\t",word,word);
    nerr=Festival2Lex(word,NULL);
    if(nerr)
      fprintf(stderr,"Error with word \"%s\" (%i)\n",word,nerr);
    else if(!ONLYPRINTPHONEMES)
      printf("\n");
    fgets(buf,sizeof(buf),stdin);
  }
  return 0;
}


int dowords(int nargs,char *args[])
{
  int a;
 
  if(nargs==1) {
    printf("Usage: festival2phone word1 [word2 [word3...]]\n");
    return -1;
  }
  for(a=1;a<nargs;a++) {
    QuitSpecialChars(args[a]);
    if(!ONLYPRINTPHONEMES)
      printf("%s\t[%s]\t",args[a],args[a]);
    if(Festival2Lex(args[a],NULL))
      fprintf(stderr,"Error with word \"%s\"\n",args[a]);
    else
      printf("\n");
  }
  return 0;
}


int main(int nargs,char *args[])
{
  if(FILTER)
    return(dofilter());
  else
    return(dowords(nargs,args));
}


